import React from 'react';
import { Form, Select } from 'antd';
import PropTypes from 'prop-types'; 
import { hasOwnProperty, isFunction, isBoolean } from './utils';

const AntFormItem = Form.Item

const tailFormItemLayout = {
  wrapperCol: {
    xs: {
      span: 24,
      offset: 0,
    },
    sm: {
      span: 16,
      offset: 8,
    },
  },
};

class Item extends React.PureComponent {
  state = {
    allFieldValues: this.getFormValues(),
  }

  componentDidMount() {
    if (this.props.formChange) {
      this.props.formChange((p, c, values) => {
        this.setState({ allFieldValues: values || this.getFormValues()})
      })
    }
  }

  getFormValues() {
    if (this.props.form) {
      return this.props.form.getFieldsValue() || {}
    }

    return {}
  }

  getActualValue(fn, defVal) {
    const val = isFunction(fn) ? fn(this.state.allFieldValues, this.props) : fn
    return val === undefined ? defVal : val
  }

  getValueFromEvent = e => {
    if (hasOwnProperty(e, 'checked')) {
      return e.checked;
    } else if (hasOwnProperty(e, 'target')) {
      return e.target.value;
    } else {
      return e;
    }
  }

  renderSelectOptions (dict) {
    return (dict || []).map(x => <Select.Option key={x.value} value={x.value}>{x.label}</Select.Option>)
  }

  renderField() {
    const {
      form,
      name,
      node,
      children,
      placeholder,
      render,
      rules,
      value,
      action,
      props,
      options,
    } = this.props || {}

    const config = {
      rules,
      initialValue: value,
      getValueFromEvent: this.getValueFromEvent,
    }

    const fieldProps = {
      placeholder: Array.isArray(placeholder) ? placeholder(this.state.allFieldValues, this.props) : placeholder,
      style: { width: '100%' },
      ...(props || {})
    }

    const Childs = Array.isArray(options) ? this.renderSelectOptions(options) : (isFunction(children) ? children() : (children || null));
    const Field = isFunction(render) ? render(this.props) : React.createElement(node, fieldProps, Childs);

    return action ? Field : form.getFieldDecorator(name, config)(Field)
  }

  render () {
    const {
      form,
      label,
      name,
      node,
      children,
      placeholder,
      render,
      rules,
      value,
      action,
      props = {},
      hidden = false,
      reportChange,
      listenChange,
      options,
      ...restProps
    } = this.props;

    const initialValue = value === undefined ? {} : { initialValue: value }
    
    const config = {
      rules,
      getValueFromEvent: this.getValueFromEvent,
      ...initialValue,
    }
    
    // 以下支持可变内容
    // 暂时只支持 hidden, disable, label 的可变
    const hiddenActual = isFunction(hidden) ? hidden(this.state.allFieldValues, this.props) : (
      isBoolean(hidden) ? hidden : false
    );
    const labelActual = isFunction(label) ? label(this.state.allFieldValues, this.props) : label
    const disabledActual = isFunction(props.disabled) ? props.disabled(this.state.allFieldValues, this.props) : (
      isBoolean(props.disabled) ? props.disabled : false
    )

    // 没有类型与组件, 生成隐藏字段
    if (name && hiddenActual === true) {
      if (form && form.getFieldDecorator) {
        form.getFieldDecorator(name, config);
      }
      return null;
    }

    const Field = this.renderField()

    if (!labelActual && !name && !action) return !hiddenActual ? Field : null;
    
    const itemProps = action ? { ...restProps, ...tailFormItemLayout, disabled: disabledActual } : { ...restProps, disabled: disabledActual }

    return !hiddenActual ? (
      <AntFormItem label={labelActual} {...itemProps}>{Field}</AntFormItem>
    ) : null
  }
};

Item.propTypes = {
  formChange: PropTypes.func,
  form: PropTypes.object,
  label: PropTypes.oneOfType([
    PropTypes.node,
    PropTypes.func,
  ]),
  placeholder: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.func,
  ]),
  name: PropTypes.string,
  node: PropTypes.oneOfType([
    PropTypes.elementType,
    PropTypes.func,
  ]),
  children: PropTypes.oneOfType([
    PropTypes.element,
    PropTypes.func,
    PropTypes.arrayOf(PropTypes.element),
  ]),
  render: PropTypes.func,
  rules: PropTypes.array,
  value: PropTypes.any,
  action: PropTypes.bool,
  props: PropTypes.object,
  options: PropTypes.array,
  hidden: PropTypes.oneOfType([
    PropTypes.bool,
    PropTypes.func,
  ]),
}

export default Item;
